import sys
import unittest
from unittest.mock import Mock, patch
import asyncio
import threading
import queue
import os

import xmlrunner
import time

import younit
from test import common
from test import unittest_utils


# @unittest.skip("skipped")
class TestNameDecorator(unittest.TestCase):
    def setUp(self):
        self.original = sys.stdout
        self.stdout_file = open('redirect.txt', 'w')
        sys.stdout = self.stdout_file

    def tearDown(self):
        sys.stdout = self.original
        self.stdout_file.close()

        expected = '\n******** STARTING TEST: test_that_name_decorator_works *********\n'
        with open('redirect.txt', 'r') as f:
            actual = f.read()

        os.remove("redirect.txt")

        self.assertEqual(expected,actual)

    @younit.test_name
    def test_that_name_decorator_works(self):
        pass

# @unittest.skip("skipped")
class TestSetClearHanging(unittest.TestCase):
    @younit.set_test_hang_alarm
    def setUp(self):
        pass

    @younit.clear_test_hang_alarm
    def tearDown(self):
        pass

    # @unittest.skip("skipped")
    def test_that_test_hangs(self):
        with self.assertRaises(younit.TestHang):
            time.sleep(2)

    # @unittest.skip("skipped")
    def test_that_test_doesnt_hang(self):
        time.sleep(0.1)

# @unittest.skip("skipped")
class TestHanging(unittest.TestCase):

    # @unittest.skip("skipped")
    @younit.test_hang_alarm
    def test_that_test_hangs(self):
        with self.assertRaises(younit.TestHang):
            time.sleep(2)

    # @unittest.skip("skipped")
    @younit.test_hang_alarm
    def test_that_test_doesnt_hang(self):
        time.sleep(0.1)



# @unittest.skip("skipped")
class TestCloseAllThreads(unittest.TestCase):

    def setUp(self):
        self.threads_to_close = []

    def tearDown(self):
        time.sleep(0.01)
        self.assertEqual(1,threading.active_count())

    @younit.close_all_threads
    def test_that_all_threads_are_closed(self):
        t = ThreadRunner()
        self.threads_to_close.append(t)
        t.start()


class AsyncioTestWithMocking(unittest.TestCase):

    async def async_setUp(self):
        self.x = younit.AsyncMock()
        await self.x()

    async def async_tearDown(self):
        await self.x()

    @younit.asyncio_test
    async def test_runs_mock(self):
        self.x.mock.assert_called_once()

class ThreadRunner(object):
    def __init__(self):
        self.q = queue.Queue()

    def start(self):
        t = threading.Thread(target=self.worker)
        t.start()

    def worker(self):
        self.q.get()


    def close(self):
        self.q.put(None)


if __name__ == '__main__':
    unittest.main(
        testRunner=xmlrunner.XMLTestRunner(output='test-reports'),
        # these make sure that some options that are not applicable
        # remain hidden from the help menu.
        failfast=False, buffer=False, catchbreak=False)



'''
younit is a collection of helpers for the :mod:`unittest` module.
'''

from .__version__ import __version__


from .main import TestHang, test_name, set_test_hang_alarm, \
    clear_test_hang_alarm, test_hang_alarm, close_all_threads, \
    asyncio_test, AsyncMock

__all__ = ["TestHang", "test_name", "set_test_hang_alarm",
           "clear_test_hang_alarm", "test_hang_alarm", "close_all_threads",
           "asyncio_test", "AsyncMock"]



from setuptools import setup
import os
import sys
PY_VER = sys.version_info

if not PY_VER >= (3, 6):
    raise RuntimeError("younit doesn't support Python earlier than 3.6")

here = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(here, 'younit', '__version__.py'), 'r') as f:
    exec(f.read(), about)

def read(fname):
    with open(os.path.join(here, fname), 'r') as f:
        return str(f.read().strip())

setup(
    name='younit',
    version=about['__version__'],
    packages=['younit'],
    description="a collection of helpers for the unittest module",
    long_description='\n\n'.join((read('README.rst'), read('CHANGELOG.rst'))),
    include_package_data=True,
    install_requires=[
        
    ],
    zip_safe=False,
    author="Jeremy Arr",
    author_email="jeremyarr@gmail.com",
    license="MIT",
    keywords=["asyncio","unittest","mock","testing"],
    url="https://github.com/jeremyarr/younit",
    classifiers = [
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.6',
    ]
)




'''
the main module where the helpers are defined
'''

import signal
import unittest
import asyncio
from unittest.mock import MagicMock


class TestHang(Exception):
    pass


def test_name(func):
    '''
    A decorator that prints the test name before
    starting a test.

    Convenient if you want to separate the output of
    different tests.

    Usage::

        class MyTestCase(unittest.TestCase):
            @test_name
            def test_this(self):
                print("im testing this")

            @test_name
            def test_that(self):
                print("im testing that")

    '''
    def inner(*args, **kwargs):
        print("\n******** STARTING TEST: {} *********".format(func.__name__))
        return func(*args, **kwargs)
    return inner


def _test_hang_handler(signum, frame):
    raise TestHang


def test_hang_alarm(func):
    '''
    A decorator that sets an alarm of 1 second before
    starting any test.

    If a test takes longer than 1 second, a :class:`TestHang`
    exception is raised.

    If a test takes less than 1 second, the alarm is cancelled.

    Usage::

        class MyTestCase(unittest.TestCase):

            @test_hang_alarm
            def test_this(self):
                time.sleep(3)

    '''
    def inner(*args, **kwargs):
        signal.signal(signal.SIGALRM, _test_hang_handler)
        signal.alarm(1)
        try:
            return func(*args, **kwargs)
        finally:
            signal.alarm(0)

    return inner


def set_test_hang_alarm(func):
    '''
    A decorator that sets an alarm of 1 second before
    starting any test.

    If a test takes longer than 1 second, a :class:`TestHang`
    exception is raised.

    Should be used during set up and in conjunction with
    :func:`clear_test_hang_alarm` during tear down.

    Usage::

        class MyTestCase(unittest.TestCase):
            @set_test_hang_alarm
            def setUp(self):
                pass

            @clear_test_hang_alarm
            def tearDown(self):
                pass
    '''

    def inner(*args, **kwargs):
        signal.signal(signal.SIGALRM, _test_hang_handler)
        signal.alarm(1)
        return func(*args, **kwargs)
    return inner


def clear_test_hang_alarm(func):
    '''
    A decorator that resets an alarm set by :func:`set_test_hang_alarm`

    Should be used during tear down and in conjunction with
    :func:`set_test_hang_alarm` during set up.

    Usage::

        class MyTestCase(unittest.TestCase):
            @set_test_hang_alarm
            def setUp(self):
                pass

            @clear_test_hang_alarm
            def tearDown(self):
                pass
    '''
    def inner(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        finally:
            signal.alarm(0)
    return inner


def close_all_threads(func):
    '''
    A decorator that closes any threads that are created
    as part of running a test.

    To use, ensure your threads are able to be closed by
    invoking a ``close()`` method on an object related to the
    thread. Then add the object to the ``self.threads_to_close``
    list.


    Usage::

        class MyTestCase(unittest.TestCase):
            def setUp(self):
                self.threads_to_close = []
                x = start_a_new_thread()
                #x is an object with a close() method
                #that closes the thread
                self.threads_to_close.append(x)

            @close_all_threads
            def test_this(self):
                y = start_a_new_thread()
                self.threads_to_close.append(y)
    '''
    def inner(self):
        try:
            return func(self)
        finally:
            [x.close() for x in self.threads_to_close]

    return inner


def asyncio_test(func):
    '''
    A decorator that runs a test as a coroutine including
    any set up and tear down coroutines.

    Usage::

        class MyTestCase(unittest.TestCase):
            async def async_setUp(self):
                pass

            async def async_tearDown(self):
                pass

            @asyncio_test
            async def test_this(self):
                pass

    '''
    def inner(self):
        async def run(self, *args, **kwargs):
            await self.async_setUp()

            try:
                return await func(self, *args, **kwargs)
            finally:
                await self.async_tearDown()

        self.loop = asyncio.new_event_loop()
        asyncio.set_event_loop(self.loop)
        self.loop.set_debug(True)

        try:
            self.loop.run_until_complete(run(self))
        finally:
            self.loop.close()

    return inner


def AsyncMock(*args, **kwargs):
    '''
    A function that can be used to mock a coroutine.

    Returns a coroutine function with a mock attribute.
    The mock attribute is a :class:`unittest.mock.MagicMock` object that records
    usage.


    Usage::

        class MyTestCase(unittest.TestCase):
            async def async_setUp(self):
                pass

            async def async_tearDown(self):
                pass

            @asyncio_test
            async def test_this(self):
                x = AsyncMock()
                await x()
                x.mock.assert_called_once()
    '''

    # thanks to miguel grinberg for this
    # https://blog.miguelgrinberg.com/post/unit-testing-asyncio-code

    m = MagicMock(*args, **kwargs)

    async def mock_coro(*args, **kwargs):
        return m(*args, **kwargs)

    mock_coro.mock = m
    return mock_coro



import unittest
from unittest.mock import Mock, patch
import asyncio

import xmlrunner

import younit
from test import common
from test import unittest_utils


# @unittest.skip("skipped")
class SimpleTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        pass

    async def async_setUp(self):
        pass

    async def async_tearDown(self):
        pass

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_this(self):
        self.assertEqual(1,1)



if __name__ == '__main__':
    unittest.main(
        testRunner=xmlrunner.XMLTestRunner(output='test-reports'),
        # these make sure that some options that are not applicable
        # remain hidden from the help menu.
        failfast=False, buffer=False, catchbreak=False)
